@@ -53,8 +53,6 @@ struct debug_classes
   unsigned int types_of_byte_code;
 };
 
-extern Lisp_Object Qredisplay, Qbuffers, Qextents, Qfaces;
-extern Lisp_Object Qwindows, Qframes, Qdevices, Qbyte_code;
 extern struct debug_classes active_debug_classes;
 
 #define DASSERT(class, desired_type, action, assertion) do		\
