@@ -26,12 +26,6 @@
 /* POSIX says that <sys/types.h> must be included (by the caller) before
    <regex.h>.  */
 
-#if !defined (_POSIX_C_SOURCE) && !defined (_POSIX_SOURCE) && defined (VMS)
-/* VMS doesn't have `size_t' in <sys/types.h>, even though POSIX says it
-   should be there.  */
-#include <stddef.h>
-#endif
-
 
 /* The following bits are used to determine the regexp syntax we
    recognize.  The not-set meaning typically corresponds to the syntax
@@ -412,38 +406,21 @@ typedef struct
 
 /* Declarations for routines.  */
 
-/* To avoid duplicating every routine declaration -- once with a
-   prototype (if we are ANSI), and once without (if we aren't) -- we
-   use the following macro to declare argument types.  This
-   unfortunately clutters up the declarations a bit, but I think it's
-   worth it.  */
-
-#ifdef __STDC__
-
-#define _RE_ARGS(args) args
-
-#else /* not __STDC__ */
-
-#define _RE_ARGS(args) ()
-
-#endif /* not __STDC__ */
-
 /* Sets the current default syntax to SYNTAX, and return the old syntax.
    You can also simply assign to the `re_syntax_options' variable.  */
-extern reg_syntax_t re_set_syntax _RE_ARGS ((reg_syntax_t syntax));
+reg_syntax_t re_set_syntax (reg_syntax_t syntax);
 
 /* Compile the regular expression PATTERN, with length LENGTH
    and syntax given by the global `re_syntax_options', into the buffer
    BUFFER.  Return NULL if successful, and an error string if not.  */
-extern CONST char *re_compile_pattern
-  _RE_ARGS ((CONST char *pattern, int length,
-             struct re_pattern_buffer *buffer));
+CONST char *re_compile_pattern (CONST char *pattern, int length,
+				struct re_pattern_buffer *buffer);
 
 
 /* Compile a fastmap for the compiled pattern in BUFFER; used to
    accelerate searches.  Return 0 if successful and -2 if was an
    internal error.  */
-extern int re_compile_fastmap _RE_ARGS ((struct re_pattern_buffer *buffer));
+int re_compile_fastmap (struct re_pattern_buffer *buffer);
 
 
 /* Search in the string STRING (with length LENGTH) for the pattern
@@ -451,31 +428,28 @@ extern int re_compile_fastmap _RE_ARGS ((struct re_pattern_buffer *buffer));
    characters.  Return the starting position of the match, -1 for no
    match, or -2 for an internal error.  Also return register
    information in REGS (if REGS and BUFFER->no_sub are nonzero).  */
-extern int re_search
-  _RE_ARGS ((struct re_pattern_buffer *buffer, CONST char *string,
-            int length, int start, int range, struct re_registers *regs));
+int re_search (struct re_pattern_buffer *buffer, CONST char *string,
+	       int length, int start, int range,
+	       struct re_registers *regs);
 
 
 /* Like `re_search', but search in the concatenation of STRING1 and
    STRING2.  Also, stop searching at index START + STOP.  */
-extern int re_search_2
-  _RE_ARGS ((struct re_pattern_buffer *buffer, CONST char *string1,
-             int length1, CONST char *string2, int length2,
-             int start, int range, struct re_registers *regs, int stop));
+int re_search_2 (struct re_pattern_buffer *buffer, CONST char *string1,
+		 int length1, CONST char *string2, int length2, int start,
+		 int range, struct re_registers *regs, int stop);
 
 
 /* Like `re_search', but return how many characters in STRING the regexp
    in BUFFER matched, starting at position START.  */
-extern int re_match
-  _RE_ARGS ((struct re_pattern_buffer *buffer, CONST char *string,
-             int length, int start, struct re_registers *regs));
+int re_match (struct re_pattern_buffer *buffer, CONST char *string,
+	      int length, int start, struct re_registers *regs);
 
 
 /* Relates to `re_match' as `re_search_2' relates to `re_search'.  */
-extern int re_match_2
-  _RE_ARGS ((struct re_pattern_buffer *buffer, CONST char *string1,
-             int length1, CONST char *string2, int length2,
-             int start, struct re_registers *regs, int stop));
+int re_match_2 (struct re_pattern_buffer *buffer, CONST char *string1,
+		int length1, CONST char *string2, int length2,
+		int start, struct re_registers *regs, int stop);
 
 
 /* Set REGS to hold NUM_REGS registers, storing them in STARTS and
@@ -490,32 +464,22 @@ extern int re_match_2
    Unless this function is called, the first search or match using
    PATTERN_BUFFER will allocate its own register data, without
    freeing the old data.  */
-extern void re_set_registers
-  _RE_ARGS ((struct re_pattern_buffer *buffer, struct re_registers *regs,
-             unsigned num_regs, regoff_t *starts, regoff_t *ends));
+void re_set_registers (struct re_pattern_buffer *buffer,
+		       struct re_registers *regs, unsigned num_regs,
+		       regoff_t *starts, regoff_t *ends);
 
 #ifdef _REGEX_RE_COMP
 /* 4.2 bsd compatibility.  */
-extern char *re_comp _RE_ARGS ((CONST char *));
-extern int re_exec _RE_ARGS ((CONST char *));
+char *re_comp (CONST char *);
+int re_exec (CONST char *);
 #endif
 
 /* POSIX compatibility.  */
-extern int regcomp _RE_ARGS ((regex_t *preg, CONST char *pattern, int cflags));
-extern int regexec
-  _RE_ARGS ((CONST regex_t *preg, CONST char *string, size_t nmatch,
-             regmatch_t pmatch[], int eflags));
-extern size_t regerror
-  _RE_ARGS ((int errcode, CONST regex_t *preg, char *errbuf,
-             size_t errbuf_size));
-extern void regfree _RE_ARGS ((regex_t *preg));
+int regcomp (regex_t *preg, CONST char *pattern, int cflags);
+int regexec (CONST regex_t *preg, CONST char *string, size_t nmatch,
+	     regmatch_t pmatch[], int eflags);
+size_t regerror (int errcode, CONST regex_t *preg, char *errbuf,
+		 size_t errbuf_size);
+void regfree (regex_t *preg);
 
 #endif /* not __REGEXP_LIBRARY_H__ */
-
-/*
-Local variables:
-make-backup-files: t
-version-control: t
-trim-versions-without-asking: nil
-End:
-*/
